#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <imsls.h>
 
int main()
{
   char **anova_row_labels;
   char *col_labels[] = {" ", "\nID", "\nDF", "\nSSQ", 
      "Mean\nsquares", "\nF", "\np-value"};
   char dashes[] = 
      "*************************************************************";
   int i, j, k, l, page_width = 132;
 
   int n = 24;         
   int n_locations = 1;
   int n_whole = 2;    
   int n_split = 2;    
   int n_sub   = 2;
   int rep[]={ 
      1, 1, 1, 1, 1, 1, 1, 1,
      2, 2, 2, 2, 2, 2, 2, 2,
      3, 3, 3, 3, 3, 3, 3, 3
   };
   int whole[]={   
      1, 1, 1, 1, 2, 2, 2, 2, 
      1, 1, 1, 1, 2, 2, 2, 2,
      1, 1, 1, 1, 2, 2, 2, 2
   };
   int split[]={    
      1, 1, 2, 2, 1, 1, 2, 2, 
      1, 1, 2, 2, 1, 1, 2, 2,  
      1, 1, 2, 2, 1, 1, 2, 2
   };
   int sub[]={    
      1, 2, 1, 2, 1, 2, 1, 2, 
      1, 2, 1, 2, 1, 2, 1, 2,  
      1, 2, 1, 2, 1, 2, 1, 2
   };
   float y[] ={    
      30.0, 40.0, 38.9, 38.2,
      41.8, 52.2, 54.8, 58.2,
      20.5, 26.9, 21.4, 25.1,
      26.4, 36.7, 28.9, 35.9,
      21.0, 25.4, 24.0, 23.3,
      34.4, 41.0, 33.0, 34.9
   };
   float grand_mean, *cv, *aov, *treatment_means;
   float *whole_plot_means, *split_plot_means;
   float *sub_plot_means, *std_err;
   int   *equal_means;  
 
   aov = imsls_f_split_split_plot(n, n_locations, n_whole, n_split, 
      n_sub, rep, whole, split, sub, y, 
      IMSLS_GRAND_MEAN, &grand_mean, 
      IMSLS_CV, &cv,
      IMSLS_TREATMENT_MEANS,  &treatment_means, 
      IMSLS_WHOLE_PLOT_MEANS, &whole_plot_means, 
      IMSLS_SPLIT_PLOT_MEANS, &split_plot_means, 
      IMSLS_SUB_PLOT_MEANS,   &sub_plot_means,
      IMSLS_STD_ERRORS,       &std_err,
      IMSLS_ANOVA_ROW_LABELS, &anova_row_labels,
      0);
 
   /* Output results. */
   imsls_page(IMSLS_SET_PAGE_WIDTH, &page_width);
 
   /* Print ANOVA table. */
   imsls_f_write_matrix("   *** ANALYSIS OF VARIANCE TABLE ***", 
      20, 6, aov, 
      IMSLS_WRITE_FORMAT, "%3.0f%3.0f%8.2f%7.2f%7.2f%7.3f",
      IMSLS_ROW_LABELS, anova_row_labels,
      IMSLS_COL_LABELS, col_labels,
      0);
 
   printf("\n\nGrand mean:    %7.3f\n", grand_mean);
   printf("Coefficient of Variation ****\n");
   printf("  Whole-Plot: %7.3f\n", cv[0]);
   printf("  Split-Plot: %7.3f\n", cv[1]);
   printf("  Sub-Plot  : %7.3f\n", cv[2]);
   l = 0;
   /*
   * Treatment Means
   */
   printf("\n\n%s\n", dashes);
   printf("Treatment Means: \n");
   for (i=0; i < n_whole; i++){
      for(j=0; j < n_split; j++){
         for(k=0; k < n_sub; k++){
            printf("  treatment[%d][%d][%d] %f \n", i, j, k, 
               treatment_means[l++]);
         }
      }
   }
   printf("\n  Standard Error for Comparing Two Treatment Means: %f ",
      std_err[3]);
   printf("\n(df=%f)\n", std_err[7]);
   equal_means = imsls_f_multiple_comparisons(n_whole*n_split*n_sub, 
      treatment_means, (int)std_err[7],
      std_err[3]/sqrt(2), 
      IMSLS_LSD,
      IMSLS_ALPHA, .05,
      0);
   printf("\n  LSD for Treatment Means (alpha=0.05)");
   imsls_i_write_matrix("  Size of Groups of Means", 1, 
      n_whole*n_split*n_sub-1, equal_means, 0);
   /*
   * Whole-plot Means
   */
   printf("\n\n%s", dashes);
   imsls_f_write_matrix("Whole-plot Means", n_whole, 1, 
      whole_plot_means, 0);
   printf("\nStandard Error for Comparing Two Whole-Plot Means: %f ",
      std_err[0]);
   printf("\n(df=%f)\n", std_err[4]);
   equal_means = imsls_f_multiple_comparisons(n_whole, 
      whole_plot_means, (int)std_err[4], std_err[0]/sqrt(2), 
      IMSLS_LSD,
      IMSLS_ALPHA, .05,
      0);
   printf("\nLSD for Whole-Plot Means (alpha=0.05) \n");
   imsls_i_write_matrix("Size of Groups of Means", 1, n_whole-1,
      equal_means, 0);
   /*
   * Split-plot Means
   */
   printf("\n\n%s",dashes);
   imsls_f_write_matrix("Split-plot Means", n_split, 1,
      split_plot_means, 0);
   printf("\nStandard Error for Comparing Two Split-Plot Means: %f ",
      std_err[1]);
   printf("\n(df=%f)\n", std_err[5]);
   equal_means = imsls_f_multiple_comparisons(n_split, 
      split_plot_means, (int)std_err[5], std_err[1]/sqrt(2), 
      IMSLS_LSD,
      IMSLS_ALPHA, .05,
      0);
   printf("\nLSD for Split-Plot Means (alpha=0.05) \n");
   imsls_i_write_matrix("Size of Groups of Means", 1, n_split-1,
      equal_means, 0);
   /*
   * Sub-plot Means
   */
   printf("\n\n%s", dashes);
   imsls_f_write_matrix("Sub-plot Means", n_sub, 1, sub_plot_means, 0);
   printf("\nStandard Error for Comparing Two Sub-Plot Means: %f ",
      std_err[2]);
   printf("\n(df=%f)\n", std_err[6]);
   equal_means = imsls_f_multiple_comparisons(n_sub, sub_plot_means, 
      (int)std_err[6], std_err[2]/sqrt(2), 
      IMSLS_LSD,
      IMSLS_ALPHA, .05,
      0);
   printf("\nLSD for Sub-Plot Means (alpha=0.05) \n");
   imsls_i_write_matrix(": Size of Groups of Means", 1, n_sub-1,
      equal_means, 0);
}
